unit RTest1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, ExtCtrls, Region, TypInfo;

type
  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    U_Btn: TSpeedButton;
    UR_Btn: TSpeedButton;
    R_Btn: TSpeedButton;
    DR_Btn: TSpeedButton;
    D_Btn: TSpeedButton;
    DL_Btn: TSpeedButton;
    L_Btn: TSpeedButton;
    UL_Btn: TSpeedButton;
    TL_Btn: TSpeedButton;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    LeftEdit: TEdit;
    TopEdit: TEdit;
    WidthEdit: TEdit;
    HeightEdit: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    GroupBox4: TGroupBox;
    CircleBtn: TSpeedButton;
    RectBtn: TSpeedButton;
    RoundRectBtn: TSpeedButton;
    RoundSquareBtn: TSpeedButton;
    SquareBtn: TSpeedButton;
    EllipseBtn: TSpeedButton;
    FontDialog: TFontDialog;
    GroupBox5: TGroupBox;
    TextEdit: TEdit;
    Font: TSpeedButton;
    TextBtn: TSpeedButton;
    GroupBox6: TGroupBox;
    ClearBtn: TSpeedButton;
    GroupBox7: TGroupBox;
    SmallerBtn: TSpeedButton;
    LargerBtn: TSpeedButton;
    HorzBtn: TSpeedButton;
    VertBtn: TSpeedButton;
    CombineRG: TRadioGroup;
    GroupBox9: TGroupBox;
    FillRG: TRadioGroup;
    FillBtn: TSpeedButton;
    GroupBox8: TGroupBox;
    BrushShape: TShape;
    ColourCB: TComboBox;
    StyleCB: TComboBox;
    procedure Move_BtnClick(Sender: TObject);
    procedure ShapeBtnClick(Sender: TObject);
    procedure FontClick(Sender: TObject);
    procedure TextBtnClick(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure ScalarClick(Sender: TObject);
    procedure CombineRGClick(Sender: TObject);
    procedure FillRGClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ColourCBChange(Sender: TObject);
    procedure StyleCBChange(Sender: TObject);
    procedure FillBtnClick(Sender: TObject);
  private
    Region1 : TRegion;
    function GetRect : TRect;
    procedure AddColourToList (const s : string);
  public
  end;

var
  Form1: TForm1;

implementation

uses RTest2;

{$R *.DFM}

function TForm1.GetRect : TRect;
begin
  Result := Rect (StrToInt (LeftEdit.Text), StrToInt (TopEdit.Text),
    StrToInt (LeftEdit.Text) + StrToInt (WidthEdit.Text),
    StrToInt (TopEdit.Text) + StrToInt (HeightEdit.Text))
end;

procedure TForm1.Move_BtnClick(Sender: TObject);
begin
  with Form2.Region do
  case (Sender as TComponent).Tag of
    0 : Move (  0, -10);
    1 : Move ( 10, -10);
    2 : Move ( 10,   0);
    3 : Move ( 10,  10);
    4 : Move (  0,  10);
    5 : Move (-10,  10);
    6 : Move (-10,  0);
    7 : Move (-10, -10);
    8 : MoveTo (0, 0)
  end;
end;

procedure TForm1.ShapeBtnClick(Sender: TObject);
var
  X : integer;
begin
  X := StrToInt (WidthEdit.Text) div 10;
  if (Sender as TComponent).Tag in [0, 4, 5] then
    HeightEdit.Text := WidthEdit.Text;

  with Form2.Region do
  begin
    case (Sender as TComponent).Tag of
      0, 1: EllipseFromRect (GetRect);
      2, 4: RectangleFromRect (GetRect);
      3, 5: with GetRect do
              RoundRect (Left, Top, Right, Bottom, X, X)
    end;

    Apply
  end;
  Form2.Repaint
end;

procedure TForm1.FontClick(Sender: TObject);
begin
  with FontDialog do
  begin
    Font.Assign (Form2.Region.Font);
    if Execute then
      Form2.Region.Font.Assign (Font)
  end
end;

procedure TForm1.TextBtnClick(Sender: TObject);
var
  R : TRect;
begin
  if TextEdit.Text <> '' then
  begin
    R := GetRect;
    Form2.Region.TextOut (R.Left, R.Top, TextEdit.Text);
    Form2.Region.Apply
//    Form2.Repaint
  end
end;

procedure TForm1.ClearBtnClick(Sender: TObject);
begin
  Form2.Region.Clear;
  Form2.Repaint
end;

procedure TForm1.ScalarClick(Sender: TObject);
const
  Scalar = 1.1;
begin
  with Form2.Region do
    case (Sender as TComponent).Tag of
      0 : Transform (1/Scalar, 1/Scalar, 0, 0);
      1 : Transform (Scalar, 1, 0, 0);
      2 : Transform (1, Scalar, 0, 0);
      3 : Transform (Scalar, Scalar, 0, 0)
    end
end;

procedure TForm1.CombineRGClick(Sender: TObject);
begin
  with Form2.Region do
    case CombineRG.ItemIndex of
      0 : CombineMode := rcIntersect;
      1 : CombineMode := rcSubtract;
      2 : CombineMode := rcUnion;
      3 : CombineMode := rcXor
    end
end;

procedure TForm1.FillRGClick(Sender: TObject);
begin
  with Form2.Region do
    case FillRG.ItemIndex of
      0 : FillMode := fmWinding;
      1 : FillMode := fmAlternate
    end
end;

procedure TForm1.AddColourToList (const S : string);
begin
  ColourCB.Items.Add (S)
end;


procedure TForm1.FormCreate(Sender: TObject);
var
  pti : PTypeInfo;
  I : Integer;
begin
// fill colour CB with permitted colours

  StyleCB.Clear;
  pti := TypeInfo (TBrushStyle);
  with GetTypeData(pti)^ do
    for I := MinValue to MaxValue do
      StyleCB.Items.AddObject (GetEnumName (pti, I), pointer(I));
  StyleCB.ItemIndex := 0;

  ColourCB.Clear;
  GetColorValues (AddColourToList);
  ColourCB.ItemIndex := 0;

// fit form to dialog components
  Region1 := TRegion.Create (self);
  with Region1 do
  begin
    Polygon ([Point (0,0), Point (700, 0), Point (700, 140), Point (500, 140),
              Point (500, 215), Point (355, 215), Point (355, 260), Point (110, 260),
              Point (110, 320), Point (0, 320)]);
    Apply
  end
end;

procedure TForm1.ColourCBChange(Sender: TObject);
var
  C : integer;
begin
  if IdentToColor (ColourCB.Text, C) then
  begin
    BrushShape.Brush.Color := C;
    Form2.Region.Brush.Assign (BrushShape.Brush)
  end
end;

procedure TForm1.StyleCBChange(Sender: TObject);
begin
  BrushShape.Brush.Style := TBrushStyle (integer(StyleCB.Items.Objects[StyleCB.ItemIndex]));
  Form2.Region.Brush.Assign (BrushShape.Brush)
end;

procedure TForm1.FillBtnClick(Sender: TObject);
begin
  Form2.Region.Fill
end;

end.
